"""
# ΦωΦ (pronounced owega)

ΦωΦ is a command-line interface for conversing with GPT models (from OpenAI),
Mistral API, and chub.ai API.
"""
from . import owega, utils
from .changelog import OwegaChangelog as _oc

__version__ = _oc.version
__all__ = ['owega', 'utils', "__version__"]



"""Handle /tokens."""
import prompt_toolkit as pt

from ..config import baseConf
from ..conversation import Conversation
from ..utils import info_print
from ..colors import clrtxt
from ..constants import OWEGA_DEFAULT_MAX_TOKENS as ODMT


# change requested tokens amount
def handle_tokens(
    temp_file: str,
    messages: Conversation,
    given: str = "",
    temp_is_temp: bool = False,
    silent: bool = False
) -> Conversation:
    """Handle /tokens.

    Command description:
        Changes the max amount of requested tokens.

    Usage:
        /tokens [max tokens]
    """
    # removes linter warning about unused arguments
    if temp_file:
        pass
    if temp_is_temp:
        pass
    given = given.strip()
    if given.isdigit():
        baseConf["max_tokens"] = int(given)
        if not silent:
            info_print(
                f'Set requested tokens to {baseConf.get("max_tokens", ODMT)}')
        return messages
    if not silent:
        info_print(
            f'Currently requested tokens: {baseConf.get("max_tokens", ODMT)}')
        info_print('How many tokens should be requested?')
    new_tokens = pt.prompt(pt.ANSI(
        '\n' + clrtxt("magenta", " tokens ") + ': '
    )).strip()
    if new_tokens.isdigit():
        baseConf["max_tokens"] = int(new_tokens)
        if not silent:
            info_print(
                f'Set requested tokens to {baseConf.get("max_tokens", ODMT)}')
    else:
        if not silent:
            info_print(
                'Invalid input, keeping current requested tokens amount')
    return messages


item_tokens = {
    "fun": handle_tokens,
    "help": "changes the max amount of requested tokens",
    "commands": ["tokens"],
}



"""Handle /frequency."""
from ..conversation import Conversation
from ..constants import OWEGA_DEFAULT_FREQUENCY_PENALTY as ODFP
from .helpers import helper_float_value


# change frequency penalty
def handle_frequency(
    temp_file: str,
    messages: Conversation,
    given: str = "",
    temp_is_temp: bool = False,
    silent: bool = False
) -> Conversation:
    f"""Handle /frequency.

    Command description:
        Sets the frequency penalty (-2.0 - 2.0, defaults {ODFP}).

    Usage:
        /frequency [frequency]
    """
    # removes linter warning about unused arguments
    _, _ = temp_file, temp_is_temp
    helper_float_value(
        "frequency_penalty",
        given,
        silent,
        -2.0,
        2.0,
        ODFP,
        "frequency penalty"
    )
    return messages


item_frequency = {
    "fun": handle_frequency,
    "help": f"sets the frequency penalty (-2.0 - 2.0, defaults {ODFP})",
    "commands": ["frequency"],
}



import logging
import os
import tempfile


def genFormatter(color: bool = False) -> logging.Formatter:
    clr = {
        'red': '\x1b[91m',
        'gre': '\x1b[92m',
        'yel': '\x1b[93m',
        'blu': '\x1b[94m',
        'mag': '\x1b[95m',
        'cyn': '\x1b[96m',
        'rst': '\x1b[0m'
    }
    logfmtstr = f'[{clr["mag"]}%(asctime)s{clr["rst"]}]'
    logfmtstr += f' [{clr["yel"]}%(filename)s{clr["rst"]}'
    logfmtstr += f':{clr["cyn"]}%(lineno)s{clr["rst"]}]'
    logfmtstr += f' [{clr["red"]}%(name)s{clr["rst"]}]'
    logfmtstr += f' [{clr["blu"]}%(levelname)s{clr["rst"]}]'
    logfmtstr += ': %(message)s'
    if not color:
        for esc in clr.values():
            logfmtstr = logfmtstr.replace(esc, '')
    return logging.Formatter(logfmtstr)


def getLoggerFile() -> str:
    # Get system temp directory
    temp_dir = tempfile.gettempdir()
    # Get username safely
    username = os.getenv('USERNAME') or os.getenv('USER') or 'user'
    # Create log filename
    logger_file = os.path.join(temp_dir, f'{username}.owega.log')
    return logger_file


def getLogger(name, debug: bool = False) -> logging.Logger:
    logger_file = getLoggerFile()
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    # Remove existing handlers to avoid duplicates
    for handler in logger.handlers[:]:
        logger.removeHandler(handler)

    # Console handler
    cons_hand = logging.StreamHandler()
    if debug:
        cons_hand.setLevel(logging.DEBUG)
    else:
        cons_hand.setLevel(logging.WARNING)
    cons_hand.setFormatter(genFormatter(color=True))
    logger.addHandler(cons_hand)

    try:
        # File handler
        # Create parent directory if it doesn't exist
        os.makedirs(os.path.dirname(logger_file), exist_ok=True)
        file_hand = logging.FileHandler(logger_file)
        file_hand.setLevel(logging.DEBUG)
        file_hand.setFormatter(genFormatter(color=False))
        logger.addHandler(file_hand)
    except (OSError, IOError):
        # If file logging fails, just continue with console logging
        pass

    return logger



"""Define helper functions for handlers."""
import prompt_toolkit as pt

from ..config import baseConf
from ..utils import info_print
from ..OwegaSession import OwegaSession as ps
from ..colors import clrtxt
# from ..conversation import Conversation

def helper_toggle(
    config_key: str,
    given: str = "",
    silent: bool = False,
    description: str = ""
) -> bool:
    """Generic handler for toggling options."""
    given = given.strip()
    if given.lower() in ["on", "true", "enable", "enabled"]:
        baseConf[config_key] = True
        if not silent:
            info_print(f"{description} enabled.")
        return True

    if given.lower() in ["off", "false", "disable", "disabled"]:
        baseConf[config_key] = False
        if not silent:
            info_print(f"{description} disabled.")
        return False

    baseConf[config_key] = (not baseConf.get(config_key, False))
    if not silent:
        status = 'enabled' if baseConf[config_key] else 'disabled'
        info_print(f"{description} {status}.")
    return baseConf[config_key]


def helper_float_value(
    config_key: str,
    given: str = "",
    silent: bool = False,
    min_val: float = 0.0,
    max_val: float = 2.0,
    default_val: float = 1.0,
    description: str = ""
) -> float:
    prev_val = baseConf.get(config_key, default_val)
    given = given.strip()
    prompt_str = '\n' + clrtxt("magenta", f" {description} ") + ': '
    new_value = default_val
    try:
        new_value = float(given)
    except ValueError:
        if not silent:
            info_print(
                f"Current {description}: "
                f"{baseConf.get(config_key, default_val)}"
            )
            info_print(
                f"New {description} value "
                f"({min_val} - {max_val}, defaults {default_val})"
            )
        try:
            if ps['float'] is not None:
                new_value = ps['float'].prompt(pt.ANSI(prompt_str)).strip()
            else:
                new_value = input(prompt_str).strip()
        except (ValueError, KeyboardInterrupt, EOFError):
            if not silent:
                info_print(f"Invalid {description}.")
            return prev_val
    new_value = float(new_value)
    baseConf[config_key] = new_value
    if new_value > max_val:
        if not silent:
            info_print(f"Value too high, capping to {max_val}")
        baseConf[config_key] = max_val
    elif new_value < min_val:
        if not silent:
            info_print(f"Value too low, capping to {min_val}")
        baseConf[config_key] = min_val
    this_val = baseConf.get(config_key, default_val)
    if not silent:
        info_print(f"Set {description} to {this_val}")
    return this_val



"""Utilities that don't fit anywhere else."""
import os
import sys
import tempfile
from typing import Literal

import json5 as json
import openai

try:
    import sounddevice as sd
    import soundfile as sf
    audio_lib = "sounddevice"
except ImportError:
    sd = None
    sf = None
    audio_lib = None

try:
    import tiktoken
except ModuleNotFoundError:
    def __internal_get_tokens(s: str) -> int:
        return len(s) // 4
else:
    def __internal_get_tokens(s: str) -> int:
        encoder = tiktoken.encoding_for_model("gpt-4")
        return len(encoder.encode(s))

from . import getLogger
from .config import baseConf, get_home_dir, info_print, debug_print
from .colors import clrtxt
from .conversation import Conversation

try:
    from rich.console import Console
    from rich.markdown import Markdown
    from rich.theme import Theme
except ModuleNotFoundError:
    def markdown_print(s: str) -> None:
        print(s)
else:
    console = Console()

    _owega_themes = {
        "system": {
            "markdown.em": "italic grey58",
        }
    }

    def owega_apply_theme():
        while (len(console._theme_stack._entries) > 1):
            console.pop_theme()

        global_theme = {}
        for theme in _owega_themes.values():
            try:
                for key, value in theme.items():
                    global_theme[key] = value
            except Exception:
                pass
        theme = Theme(global_theme)
        console.push_theme(theme)

    def owega_reload_themes():
        for theme_name in [
            key
            for key in baseConf.keys()
            if (key.startswith("theme.") or (key == "theme"))
        ]:
            _owega_themes[theme_name] = baseConf.get(theme_name, {}).copy()

    def markdown_print(s: str) -> None:
        if baseConf.get("fancy", False):
            if (len(console._theme_stack._entries) <= 1):
                owega_reload_themes()
                owega_apply_theme()
            console.print(Markdown(s))
        else:
            print(s)


def set_term_title(new_title: str) -> None:
    print(f"\033]0;{new_title}\a", end='')


def get_temp_file() -> str:
    """Get a temp file location."""
    tmp = tempfile.NamedTemporaryFile(
        prefix="owega_temp.",
        suffix=".json",
        delete=False
    )
    filename = tmp.name
    tmp.close()
    return filename


def command_text(msg) -> str:
    """Print a command message."""
    return ' ' + clrtxt("red", "COMMAND") + ": " + msg


def success_msg() -> str:
    """Return the standard success message."""
    return '  ' + clrtxt("cyan", " INFO ") + ": Owega exited successfully!"


def genconfig(conf_path="") -> None:
    """Generate the config file if it doesn't exist already."""
    _ = conf_path
    conf_dir = os.path.expanduser('~/.config/owega')
    conf_file = os.path.join(conf_dir, 'config.json5')
    conf_file_api = os.path.join(conf_dir, 'api.json5')
    if not os.path.exists(conf_dir):
        os.mkdir(conf_dir)
    else:
        if not os.path.isdir(conf_dir):
            info_print('Error: "~/.config/owega" is not a directory.')
    config_noapi = {}
    config_api = {}
    for key, val in baseConf.items():
        if "api" in key.lower():
            if not isinstance(val, str):
                val = ""
            config_api[key] = val
        else:
            config_noapi[key] = val

    if not os.path.exists(conf_file_api):
        with open(conf_file_api, "w") as f:
            f.write('// vim: set ft=json5:\n')
            f.write(str(json.dumps(config_api, indent=4)))
        info_print(f"saved api keys as ~/.config/owega/api.json5 !")

    should_write = True
    if os.path.isfile(conf_file):
        should_write = False
        print(
            clrtxt('red', ' WARNING ')
            + ": YOU ALREADY HAVE A CONFIG FILE AT "
            + "~/.config/owega/config.json5"
        )
        print(
            clrtxt('red', ' WARNING ')
            + ": DO YOU REALLY WANT TO OVERWRITE IT???")

        inps = clrtxt("red", "   y/N   ") + ': '
        inp = input(inps).lower().strip()
        if inp:
            if inp[0] == 'y':
                should_write = True

    if should_write:
        with open(conf_file, "w") as f:
            f.write('// vim: set ft=json5:\n')
            f.write(str(json.dumps(config_noapi, indent=4)))
        info_print("saved configuration as ~/.config/owega/config.json5 !")
        return

    info_print("Sorry, not sorry OwO I won't let you nuke your config file!!!")


def play_opus(location: str) -> None:
    """Play an OPUS audio file."""
    _ = location
    if audio_lib == "sounddevice" and sf is not None and sd is not None:
        try:
            data, fs = sf.read(location)
            sd.play(data, fs)
            sd.wait()  # wait until file is done playing
        except Exception as e:
            info_print(f"Could not play audio: {e}")
    else:
        info_print("Could not play audio, missing audio library.")


def tts_to_opus(
    loc: str,
    text: str,
    voice: Literal['alloy', 'echo', 'fable', 'onyx', 'nova', 'shimmer'] = 'nova'
) -> None:
    """Generate TTS audio from given text and save it to an opus file."""
    tts_answer = openai.audio.speech.create(
        model='tts-1',
        voice=voice,
        input=text
    )
    tts_answer.write_to_file(loc)


def play_tts(
    text: str,
    voice: Literal['alloy', 'echo', 'fable', 'onyx', 'nova', 'shimmer'] = 'nova'
) -> None:
    """Generate TTS audio from given text and play it."""
    tmpfile = tempfile.NamedTemporaryFile(
        prefix="owegatts.",
        suffix=".opus",
        delete=False
    )
    tmpfile.close()
    tts_to_opus(tmpfile.name, text, voice)
    play_opus(tmpfile.name)
    os.remove(tmpfile.name)


def estimated_tokens(ppt: str, messages: Conversation, functions: list) -> int:
    """Estimate the history tokens."""
    try:
        total_tokens = 0

        # Count prompt tokens if provided
        if ppt:
            total_tokens += __internal_get_tokens(ppt)

        # Count message tokens
        for msg in messages.get_messages():
            content = msg.get('content', '')
            if isinstance(content, str):
                total_tokens += __internal_get_tokens(content)
            elif isinstance(content, list):
                # Handle vision messages
                for item in content:
                    if item.get('type') == 'text':
                        total_tokens += __internal_get_tokens(
                            item.get('text', '')
                        )
                    elif item.get('type') == 'image_url':
                        # Rough estimate for images (varies by size)
                        total_tokens += 85  # Low detail estimate

        # Count function/tool tokens (rough estimate)
        if functions:
            for func in functions:
                # Count function definition tokens
                func_str = str(func.get('function', {}))
                total_tokens += __internal_get_tokens(func_str)

        # Add overhead for message formatting (~4 tokens per message)
        total_tokens += len(messages.get_messages()) * 4
        return total_tokens
    except Exception as e:
        logger = getLogger.getLogger(__name__, baseConf.get("debug", False))
        logger.info("An error has occured while estimating tokens:")
        logger.info(e)
        return 0


def estimated_cost(
    input_tokens: int, model: str, output_tokens: int = 4096, provider: str = ""
) -> float:
    """Calculate estimated cost based on token counts."""
    from .pricing import get_model_pricing

    input_price, output_price = get_model_pricing(model, provider)

    if input_price == 0 and output_price == 0:
        return 0.0

    return (input_tokens * input_price) + (output_tokens * output_price)


def estimated_tokens_and_cost(
    ppt: str,
    messages: Conversation,
    functions: list,
    model: str,
    output_tokens: int = 4096,
    provider: str = ""
) -> tuple[int, float]:
    """Estimate tokens and cost with provider support."""
    input_tokens = estimated_tokens(ppt, messages, functions)
    cost = estimated_cost(input_tokens, model, output_tokens, provider)
    return (input_tokens, cost)


def do_quit(
    msg="", value=0, temp_file="", is_temp=False, should_del=False
) -> None:
    """Quit and delete the given file if exists."""
    if (temp_file):
        if should_del:
            try:
                os.remove(temp_file)
            except OSError:
                pass
        else:
            if is_temp:
                try:
                    with open(temp_file, 'r') as f:
                        contents = json.loads(f.read())
                        if isinstance(contents, dict):
                            if not (
                                (len(contents.get("messages", [])) > 0)
                                or (len(contents.get("souvenirs", [])) > 0)
                            ):
                                os.remove(temp_file)
                except (OSError, ValueError):
                    pass
    if (msg):
        print()
        print(msg)
    sys.exit(value)


def print_help(commands_help=None) -> None:
    """Print the command help."""
    if commands_help is None:
        commands_help = {}
    commands = list(commands_help.keys())
    longest = 0
    for command in commands:
        if len(command) > longest:
            longest = len(command)
    longest += 1
    print()
    info_print(
        "Enter your question after the user prompt, "
        + "and it will be answered by OpenAI")
    info_print("other commands are:")
    for cmd, hstr in commands_help.items():
        command = '/' + cmd
        info_print(f"   {command:>{longest}}  - {hstr}")
    print()



"""Handle /fancy."""
from ..conversation import Conversation
from .helpers import helper_toggle


def handle_fancy(
    temp_file: str,
    messages: Conversation,
    given: str = "",
    temp_is_temp: bool = False,
    silent: bool = False
) -> Conversation:
    """Handle /fancy.

    Command description:
        Toggles fancy printing (requires python-rich).

    Usage:
        /fancy [on/true/enable/enabled/off/false/disable/disabled]
    """
    # removes linter warning about unused arguments
    _, _ = temp_file, temp_is_temp
    helper_toggle("fancy", given, silent, "Fancy printing")
    return messages


item_fancy = {
    "fun": handle_fancy,
    "help": "toggles fancy printing (requires python-rich)",
    "commands": ["fancy"],
}



"""Handle /presence."""
from ..conversation import Conversation
from ..constants import OWEGA_DEFAULT_PRESENCE_PENALTY as ODPP
from .helpers import helper_float_value


# change presence penalty
def handle_presence(
    temp_file: str,
    messages: Conversation,
    given: str = "",
    temp_is_temp: bool = False,
    silent: bool = False
) -> Conversation:
    f"""Handle /presence.

    Command description:
        Sets the presence penalty (-2.0 - 2.0, defaults {ODPP}).

    Usage:
        /presence [presence]
    """
    # removes linter warning about unused arguments
    _, _ = temp_file, temp_is_temp
    helper_float_value(
        "presence_penalty",
        given,
        silent,
        -2.0,
        2.0,
        ODPP,
        "presence penalty"
    )
    return messages


item_presence = {
    "fun": handle_presence,
    "help": f"sets the presence penalty (-2.0 - 2.0, defaults {ODPP})",
    "commands": ["presence"],
}



"""Main bottom toolbar for TUI."""
import shutil
from typing import Any

from prompt_toolkit import styles

from ..changelog import OwegaChangelog
from ..config import baseConf


# bottom toolbar and style for prompt_toolkit
def main_bottom_toolbar(what: str = "toolbar") -> Any:
    """Return the bottom style or toolbar."""
    if what == "style":
        msd = {
            'red':     '#000000 bg:#FF0000',  # noqa: E241
            'green':   '#000000 bg:#00FF00',  # noqa: E241
            'blue':    '#000000 bg:#0000FF',  # noqa: E241
            'yellow':  '#000000 bg:#FFFF00',  # noqa: E241
            'magenta': '#000000 bg:#FF00FF',  # noqa: E241
            'cyan':    '#000000 bg:#00FFFF',  # noqa: E241
            'white':   '#000000 bg:#FFFFFF',  # noqa: E241
        }
        msd['bottom-toolbar'] = msd['white']
        msd['bottom-even'] = msd['magenta']
        msd['bottom-odd'] = msd['cyan']
        msd['bottom-on'] = msd['green']
        msd['bottom-off'] = msd['red']
        main_style = styles.Style.from_dict(msd)
        return main_style

    # noinspection PyPep8Naming
    class tr:
        def __init__(self) -> None:
            self.table = []
            self.count = 0
            self.wpos = 0

        def add(self, prefix="/", txt="", color="yellow") -> None:
            new_class = "white"
            if not isinstance(txt, str):
                txt = str(txt)
            to_add = []
            to_add_count = 0
            if self.count:
                to_add.append((
                    "class:blue",
                    " - "
                ))
                to_add_count += len(' - ')
            if prefix:
                if txt:
                    prefix = prefix + ": "
                to_add.append(tuple([
                    f"class:{new_class}",
                    prefix
                ]))
                to_add_count += len(prefix)
            if txt:
                to_add.append((
                    f"class:{color}",
                    txt
                ))
                to_add_count += len(txt)

            term_w = shutil.get_terminal_size()[0]
            if self.count:
                if (self.wpos + to_add_count) > term_w:
                    self.newline()
                    to_add = to_add[1:]
                    to_add_count -= 3

            for e in to_add:
                self.table.append(e)

            self.count += 1
            self.wpos += to_add_count

        def newline(self) -> None:
            self.table.append(("class:blue", '\n'))
            self.count = 0
            self.wpos = 0

    to_ret = tr()
    to_ret.add(f"v{OwegaChangelog.version}")
    to_ret.add("model", baseConf.get("model", "unknown"))
    to_ret.add(
        "cmds",
        "ON" if baseConf.get("commands") else "OFF",
        "bottom-on" if baseConf.get("commands") else "bottom-off"
    )
    to_ret.add(
        "web",
        "ON" if baseConf.get("web_access") else "OFF",
        "bottom-on" if baseConf.get("web_access") else "bottom-off"
    )
    to_ret.add(
        "LTS",
        "ON" if baseConf.get("lts_enabled") else "OFF",
        "bottom-on" if baseConf.get("lts_enabled") else "bottom-off"
    )
    to_ret.add("tokens", baseConf.get("max_tokens", "unknown"))
    to_ret.add(
        "time",
        "ON" if baseConf.get("time_awareness") else "OFF",
        "bottom-on" if baseConf.get("time_awareness") else "bottom-off"
    )
    to_ret.add(
        "fancy",
        "ON" if baseConf.get("fancy") else "OFF",
        "bottom-on" if baseConf.get("fancy") else "bottom-off"
    )
    to_ret.add(
        "estm",
        "ON" if baseConf.get("estimation") else "OFF",
        "bottom-on" if baseConf.get("estimation") else "bottom-off"
    )
    to_ret.add(
        "TTS",
        "ON" if baseConf.get("tts_enabled") else "OFF",
        "bottom-on" if baseConf.get("tts_enabled") else "bottom-off"
    )
    to_ret.add("temp", baseConf.get("temperature", "unknown"))
    to_ret.add("top_p", baseConf.get("top_p", "unknown"))
    to_ret.add("freq.plty", baseConf.get("frequency_penalty", "unknown"))
    to_ret.add("pr.plty", baseConf.get("presence_penalty", "unknown"))
    return to_ret.table



"""Image generation functions."""
import json
from openai import OpenAI, OpenAIError

from .functions import Functions
from ..config import baseConf

ImageGenerators = Functions()


def __gen_image(*args, **kwargs) -> str:
    image_prompt = ""
    if len(args) > 0:
        image_prompt = args[0]
    image_prompt = kwargs.get("image_prompt", image_prompt)
    rdict = {}
    if not image_prompt:
        rdict["function_status"] = "No prompt provided, no image generated"
        return json.dumps(rdict)

    api_key_envvar = True
    try:
        client = OpenAI()
    except OpenAIError:
        api_key_envvar = False
        client = OpenAI(api_key='')

    if baseConf.get("api_key", "").startswith("sk-"):
        client.api_key = baseConf.get("api_key", "")
    else:
        if not api_key_envvar:
            rdict["function_status"] = "OpenAI API key invalid"
            return json.dumps(rdict)

    response = client.images.generate(
        model="dall-e-3",
        prompt=image_prompt,
        size="1024x1024",
        quality="standard",
        n=1,
    )

    image = response.data[0]
    image_url = image.url
    image_newprompt = image.revised_prompt
    rdict["generated_image"] = image_url
    rdict["revised_prompt"] = image_newprompt

    rdict["function_status"] = "Image successfully generated."
    return json.dumps(rdict)


__gen_image_desc = {
    "name": "gen_image",
    "description":
        "Generates an image from given prompt, and returns the image as an url,"
        " and the prompt after it's been revised."
    ,
    "parameters": {
        "type": "object",
        "properties": {
            "image_prompt": {
                "type": "string",
                "description":
                    "the prompt to generate an image with. will use DALL-E 3"
            }
        },
        "required": ["image_prompt"],
    },
}


ImageGenerators.addFunction(__gen_image, __gen_image_desc)



"""Handle /commands."""
from ..conversation import Conversation
from ..OwegaFun import existingFunctions
from .helpers import helper_toggle


# enables/disables command execution
def handle_web(
    temp_file: str,
    messages: Conversation,
    given: str = "",
    temp_is_temp: bool = False,
    silent: bool = False
) -> Conversation:
    """Handle /web.

    Command description:
        Toggles web access.

    Usage:
        /web [on/true/enable/enabled/off/false/disable/disabled]
    """
    # removes linter warning about unused arguments
    _, _ = temp_file, temp_is_temp
    status = helper_toggle("web_access", given, silent, "Web access")
    if status:
        existingFunctions.enableGroup("utility.user")
    else:
        existingFunctions.disableGroup("utility.user")

    return messages


item_web = {
    "fun": handle_web,
    "help": "toggles web access",
    "commands": ["web"],
}



"""Handle /history."""
from ..conversation import Conversation


# shows chat history
def handle_history(
    temp_file: str,
    messages: Conversation,
    given: str = "",
    temp_is_temp: bool = False,
    silent: bool = False
) -> Conversation:
    """Handle /history.

    Command description:
        Prints the conversation history.

    Usage:
        /history
    """
    # removes linter warning about unused arguments
    if temp_file:
        pass
    if given:
        pass
    if temp_is_temp:
        pass
    if not silent:
        messages.print_history()
    return messages


item_history = {
    "fun": handle_history,
    "help": "prints the conversation history (use /reprint for fancy printing)",
    "commands": ["history"],
}



"""Owega base configuration module."""

import os

import json5 as json
from ..constants import (
    OWEGA_DEFAULT_PROMPT,
    OWEGA_DEFAULT_MAX_TOKENS,
    OWEGA_DEFAULT_TEMPERATURE,
    OWEGA_DEFAULT_TOP_P,
    OWEGA_DEFAULT_FREQUENCY_PENALTY,
    OWEGA_DEFAULT_PRESENCE_PENALTY
)
from ..colors import clrtxt

baseModels = [
    "openai:gpt-4o",
    "mistral:open-mixtral-8x22b",
    "mistral:mistral-large-latest",
    "chub:mixtral",
    "chub:mars",
    "chub:mercury",
    "anthropic:claude-3-7-sonnet-latest",
    "openrouter:deepseek/deepseek-chat-v3-0324:free",
    "xai:grok-2-1212",
    "custom:deepseek/deepseek-chat-v3-0324:free@https://openrouter.ai/api/v1"
]
baseConf = {
    "api_key": "",  # OpenAI API key
    "organization": "",  # OpenAI organization
    "mistral_api": "",  # Mistral API key
    "chub_api": "",  # Chub Venus API key
    "claude_api": "",  # Anthropic Claude API key
    "xai_api": "",  # xAI API key
    "openrouter_api": "",  # OpenRouter API key
    "custom_api": "",  # Custom API key (for custom endpoint)
    "custom_endpoint": "",  # Custom endpoint (should be OpenAI-compatible)
    "default_prompt": (  # Default context prompt
        OWEGA_DEFAULT_PROMPT
    ),
    "model": baseModels[0],  # default model (first entry in baseModels)
    "temperature": OWEGA_DEFAULT_TEMPERATURE,  # AI Temperature (randomness)

    # AI generation parameters (top_p and penalties)
    "top_p": OWEGA_DEFAULT_TOP_P,
    "frequency_penalty": OWEGA_DEFAULT_FREQUENCY_PENALTY,
    "presence_penalty": OWEGA_DEFAULT_PRESENCE_PENALTY,

    "max_tokens": OWEGA_DEFAULT_MAX_TOKENS,  # Max tokens in response
    "available_models": baseModels,  # Available models
    "debug": False,  # Debug mode
    "commands": False,  # Command execution
    "web_access": False,  # Web access
    "lts_enabled": False,  # Long-term souvenirs feature
    "time_awareness": False,  # Save current date and time with each user msg
    "estimation": True,  # Cost and tokens estimation
    "tts_enabled": False,  # Default TTS status
    "fancy": True,  # Fancy print (requires python-rich)

    # Pre/post message injections
    "pre_user": "",
    "post_user": "",
    "pre_assistant": "",
    "post_assistant": "",
    "pre_system": "",
    "post_system": "",

    # Pre/post history injections: list of message dicts
    "pre_history": [],
    "post_history": [],
}


def get_home_dir() -> str:
    """Get the user home directory, cross-platform."""
    return os.path.expanduser('~')


def debug_print(text: str) -> None:
    """
    Print a message if debug is enabled.

    Parameters
    ----------
    text : str
        The text to print if debug is enabled.
    """
    if baseConf.get("debug", False):
        print(' ' + clrtxt("magenta", " DEBUG ") + ": " + text)


def info_print(msg) -> None:
    """Print an info message."""
    print('  ' + clrtxt("cyan", " INFO ") + ": ", end='')
    print(msg)


def get_conf(conf_path: str = "") -> None:
    """
    Load the config from a config file.

    Parameters
    ----------
    conf_path : str, optional
        The path to the config file to load.
    """

    config_files = []

    def this_add_file(path):
        expanded = os.path.expanduser(path)
        if os.path.isfile(expanded):
            if expanded not in config_files:
                config_files.append(expanded)
        elif os.path.isdir(expanded):
            this_add_dir(expanded)

    def this_add_dir(path):
        expanded = os.path.expanduser(path)
        if os.path.isdir(expanded):
            for root, dirs, files in os.walk(expanded):
                _ = dirs
                for file in files:
                    if file.endswith('.json') or file.endswith('.json5'):
                        this_add_file(os.path.join(root, file))
        elif os.path.isfile(expanded):
            this_add_file(expanded)

    def this_add_generic(path):
        if not path:
            return

        expanded = os.path.expanduser(path)

        if not os.path.exists(expanded):
            return

        if os.path.isfile(expanded):
            this_add_file(expanded)
        elif os.path.isdir(expanded):
            this_add_dir(expanded)

    this_add_file("~/.owega.json")
    this_add_dir("~/.config/owega")

    this_add_generic(conf_path)

    config_files.sort()
    debug_print(f"{config_files}")

    if not config_files:
        return

    new_debug_status = baseConf.get('debug', False)
    for file in config_files:
        debug_print(f"Loading {file}...")
        with open(file) as f:
            try:
                conf_dict = json.load(f)
            except ValueError as e:
                debug_print("Error, file dismissed.")
                debug_print(f"Error: {e}")
                continue
            if isinstance(conf_dict, dict):
                for k, v in conf_dict.items():
                    try:
                        if k == "debug":
                            new_debug_status = v
                        else:
                            baseConf[k] = v
                    except ValueError:
                        pass

    baseConf["debug"] = new_debug_status


def list_models() -> None:
    """
    List available models.

    Notes
    -----
    This function only prints from the config, it does not return anything.
    Config should have been loaded before calling this function.
    """
    info_print("Available models:")
    for index, model in enumerate(baseConf.get("available_models", [])):
        info_print(f"    [{index}]: {model}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Setup config for installing the package."""

from setuptools import setup


def get_long_description():
    desc = open('README.md').read()
    try:
        changelog = open('CHANGELOG').read()
        desc += '\n\n'
        desc += "## CHANGELOG: "
        desc += '\n```\n'
        desc += changelog
        desc += '\n```\n'
    except FileNotFoundError:
        try:
            from owega.changelog import OwegaChangelog as oc
            changelog = oc.log
            desc += '\n\n'
            desc += "## CHANGELOG: "
            desc += '\n```\n'
            desc += changelog
            desc += '\n```\n'
        except ModuleNotFoundError:
            pass
    return desc


setup()



"""Handle /time."""
from ..conversation import Conversation
from .helpers import helper_toggle


def handle_time(
    temp_file: str,
    messages: Conversation,
    given: str = "",
    temp_is_temp: bool = False,
    silent: bool = False
) -> Conversation:
    """Handle /time.

    Command description:
        Toggles sending the date and time with each message.
        (time-aware mode)

    Usage:
        /time [on/true/enable/enabled/off/false/disable/disabled]
    """
    # removes linter warning about unused arguments
    _, _ = temp_file, temp_is_temp
    helper_toggle("time_awareness", given, silent, "Time-aware mode")
    return messages


item_time = {
    "fun": handle_time,
    "help": "toggles sending the date and time with each message (time-aware)",
    "commands": ["time"],
}



"""Handle /temperature."""
from ..conversation import Conversation
from ..constants import OWEGA_DEFAULT_TEMPERATURE as ODT
from .helpers import helper_float_value


# change temperature
def handle_temperature(
    temp_file: str,
    messages: Conversation,
    given: str = "",
    temp_is_temp: bool = False,
    silent: bool = False
) -> Conversation:
    f"""Handle /temperature.

    Command description:
        Sets the temperature (0.0 - 2.0, defaults {ODT}).

    Usage:
        /temperature [temperature]
    """
    # removes linter warning about unused arguments
    _, _ = temp_file, temp_is_temp
    helper_float_value(
        "temperature",
        given,
        silent,
        0.0,
        2.0,
        ODT,
        "temperature"
    )
    return messages


item_temperature = {
    "fun": handle_temperature,
    "help": f"sets the temperature (0.0 - 2.0, defaults {ODT})",
    "commands": ["temperature"],
}
